/*******************************************************************************
 * Copyright (c) 2012, 2013 Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.objmap.graph.annotations.factories;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.exception.DatabaseException;
import org.simantics.objmap.bidirectional.IBidirectionalMappingRule;
import org.simantics.objmap.graph.annotations.RelatedGetObj;
import org.simantics.objmap.graph.annotations.RelatedSetObj;
import org.simantics.objmap.graph.rules.MappedElementRule;
import org.simantics.objmap.graph.rules.domain.RelatedObjectAccessor;
import org.simantics.objmap.graph.rules.factory.IGetSetRuleFactory;
import org.simantics.objmap.graph.rules.range.GetSetObjectAccessor;


/**
 * Rule factory for mapped object using Getter/Setter-methods.
 * 
 * @author Marko Luukkainen <marko.luukkainen@vtt.fi>
 *
 */
public class RelatedGetSetObjRuleFactory<Range> implements IGetSetRuleFactory<Resource,Range> {
	
	@Override
	public IBidirectionalMappingRule<Resource,Range> create(ReadGraph g, Annotation annotation,
			Method getter, Method setter)
			throws DatabaseException {
		RelatedGetObj getterAnn = (RelatedGetObj)annotation;
		return new MappedElementRule<Resource,Range>(new RelatedObjectAccessor(g.getResource(getterAnn.value())),
					new GetSetObjectAccessor<Range,Range>(getter, setter));
	}
	
	@Override
	public boolean isSetter(Annotation getterAnnotation, Annotation annotation) {
		RelatedGetObj getterAnn = (RelatedGetObj)getterAnnotation;
		RelatedSetObj setterAnn = (RelatedSetObj)annotation;
		return getterAnn.value().equals(setterAnn.value());
	}
	
	

}
