/*******************************************************************************
 * Copyright (c) 2012, 2013 Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.objmap.graph.rules.range;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.simantics.objmap.exceptions.MappingException;


/**
 * Accessor for mapped objects. Uses two methods:
 * - Getter: returns the current object.
 * - Setter: sets the current object. The object may be null.
 * 
 * @author Marko Luukkainen <marko.luukkainen@vtt.fi>
 *
 * @param <T>
 */
public class GetSetObjectAccessor<Range,T>  implements IRangeAccessor<Range,T> {
	
	private Method getter;
	private Method setter;
	
	
	public GetSetObjectAccessor(Method getter, Method setter) {
		this.getter = getter;
		this.setter = setter;
		
	}
	
	@SuppressWarnings("unchecked")
	public T get(Range element) throws MappingException {
		try {
			return (T) getter.invoke(element);
		} catch (IllegalArgumentException e) {
			throw new MappingException(e);
		} catch (IllegalAccessException e) {
			throw new MappingException(e);
		} catch (InvocationTargetException e) {
			throw new MappingException(e.getCause());
		}
	};

	@Override
	public boolean set(Range element, T value)
			throws MappingException {	
		try {
			setter.invoke(element, value);
		} catch (IllegalArgumentException e) {
			throw new MappingException(e);
		} catch (IllegalAccessException e) {
			throw new MappingException(e);
		} catch (InvocationTargetException e) {
			throw new MappingException(e.getCause());
		}
		return true;
		
	}
}
