/*******************************************************************************
 * Copyright (c) 2012, 2013 Association for Decentralized Information Management in
 * Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.objmap.structural.annotations.factories;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

import org.simantics.db.ReadGraph;
import org.simantics.db.exception.DatabaseException;
import org.simantics.objmap.bidirectional.IBidirectionalMappingRule;
import org.simantics.objmap.graph.annotations.RelatedElementsAdd;
import org.simantics.objmap.graph.annotations.RelatedElementsGet;
import org.simantics.objmap.graph.annotations.RelatedElementsRem;
import org.simantics.objmap.graph.rules.MappedElementsRule;
import org.simantics.objmap.graph.rules.factory.ICollectionRuleFactory;
import org.simantics.objmap.graph.rules.range.CollectionAccessor;
import org.simantics.objmap.structural.StructuralResource;
import org.simantics.objmap.structural.rules.domain.RelatedObjectsAccessor;


public class RelatedElementsRuleFactory2<Range> implements ICollectionRuleFactory<StructuralResource,Range> {
	
	@Override
	public IBidirectionalMappingRule<StructuralResource, Range> create(ReadGraph g, Annotation annotation,
			Method getter, Method adder, Method remover)
			throws DatabaseException {
		RelatedElementsGet getterAnn = (RelatedElementsGet)annotation;
		return new MappedElementsRule<StructuralResource,Range>(new RelatedObjectsAccessor(g.getResource(getterAnn.value()),getterAnn.composition(),false,true,false),
					new CollectionAccessor<Range,Range>(getter, adder, remover));
	}
	
	@Override
	public boolean isAdder(Annotation getterAnnotation, Annotation annotation) {
		RelatedElementsGet getterAnn = (RelatedElementsGet)getterAnnotation;
		RelatedElementsAdd adderAnn = (RelatedElementsAdd)annotation;
		return getterAnn.value().equals(adderAnn.value());
	}
	
	@Override
	public boolean isRemover(Annotation getterAnnotation, Annotation annotation) {
		RelatedElementsGet getterAnn = (RelatedElementsGet)getterAnnotation;
		RelatedElementsRem adderAnn = (RelatedElementsRem)annotation;
		return getterAnn.value().equals(adderAnn.value());
	}

}
